﻿using System;
using Microsoft.Win32;
using System.Linq;
using System.Collections.Generic;

namespace DirectDrawFixer
{
    public class RegOP
    {
        class RegKeyGuard : IDisposable
        {
            RegistryKey _key;

            public RegKeyGuard(RegistryKey key)
            {
                _key = key;
            }

            public void Dispose()
            {
                if (_key != null)
                    _key.Close();
            }
        }

        public static string wownode = "";

        public struct ConfigKey
        {
            public string SubKey;
            public UInt32 ID;
            public string Name;
            public UInt32 Flags;
        }

        public static bool ReadRecent(out string name, out UInt32 id)
        {
            var localMachine = Registry.LocalMachine;
            var recentKey = localMachine.OpenSubKey(@"SOFTWARE" + wownode + @"\Microsoft\DirectDraw\MostRecentApplication", false);
            using (new RegKeyGuard(recentKey))
            {
                string appName = recentKey.GetValue("Name") as string;
                if (appName != null)
                {
                    UInt32 appID = (UInt32)(Int32)recentKey.GetValue("ID");
                    name = appName;
                    id = appID;
                    return true;
                }
                else
                {
                    name = null;
                    id = 0;
                    return false;
                }
            }
        }

        private static UInt32 GetBinary(byte[] val)
        {
            if (val.Length != 4)
                throw new ArgumentException();
            return (UInt32)val[0] + (UInt32)val[1] * 256 + (UInt32)val[2] * 65536 + (UInt32)val[3] * 16777216;
        }

        private static byte[] SetBinary(UInt32 val)
        {
            byte[] r = new byte[4];
            r[0] = (byte)(val % 256);
            val /= 256;
            r[1] = (byte)(val % 256);
            val /= 256;
            r[2] = (byte)(val % 256);
            val /= 256;
            r[3] = (byte)val;
            return r;
        }

        private static List<ConfigKey> ReadAllConfigs()
        {
            var localMachine = Registry.LocalMachine;
            var configKeys = localMachine.OpenSubKey(@"SOFTWARE" + wownode + @"\Microsoft\DirectDraw\Compatibility", false);
            using (new RegKeyGuard(configKeys))
            {
                string[] configKeyNames = configKeys.GetSubKeyNames();

                List<ConfigKey> r = new List<ConfigKey>();
                foreach (var subKeyName in configKeyNames)
                {
                    try
                    {
                        var configKey = configKeys.OpenSubKey(subKeyName);
                        using (new RegKeyGuard(configKey))
                        {
                            ConfigKey ck;
                            ck.SubKey = subKeyName;
                            ck.Name = (string)configKey.GetValue("Name");
                            ck.ID = GetBinary((byte[])configKey.GetValue("ID"));
                            ck.Flags = GetBinary((byte[])configKey.GetValue("Flags"));
                            r.Add(ck);
                        }
                    }
                    finally
                    {
                    }
                }
                return r;
            }
        }

        List<ConfigKey> _configs;

        public RegOP()
        {
            RefreshList();
        }

        public void RefreshList()
        {
            _configs = ReadAllConfigs();
        }

        bool FindSpecified(string name, UInt32 id)
        {
            name = name.ToLower();
            if (_configs.Count(x => x.Name.ToLower() == name && x.ID == id) > 0)
                return true;
            else
                return false;
        }

        public List<string> GetList()
        {
            List<string> r = new List<string>();
            _configs.ForEach(x => r.Add(x.SubKey));
            return r;
        }

        public ConfigKey GetItem(string subKey)
        {
            return _configs.Find(x => x.SubKey == subKey);
        }

        public bool AddConfig(ConfigKey ck)
        {
            if (FindSpecified(ck.Name, ck.ID))
                return false;

            var localMachine = Registry.LocalMachine;
            var configKeys = localMachine.OpenSubKey(@"SOFTWARE" + wownode + @"\Microsoft\DirectDraw\Compatibility", true);
            using (new RegKeyGuard(configKeys))
            {
                var subKey = configKeys.CreateSubKey(ck.SubKey);
                using (new RegKeyGuard(subKey))
                {
                    subKey.SetValue("Name", ck.Name, RegistryValueKind.String);
                    subKey.SetValue("ID", SetBinary(ck.ID), RegistryValueKind.Binary);
                    subKey.SetValue("Flags", SetBinary(ck.Flags), RegistryValueKind.Binary);
                    return true;
                }
            }
        }

        public bool RemoveConfig(string subKey)
        {
            var localMachine = Registry.LocalMachine;
            var configKeys = localMachine.OpenSubKey(@"SOFTWARE" + wownode + @"\Microsoft\DirectDraw\Compatibility", true);

            using (new RegKeyGuard(configKeys))
            {
                configKeys.DeleteSubKey(subKey);
                return true;
            }
        }
    }
}
